/*******************************************************************************
 * @author Karolina Krolik <a href='mailto:k.krolik@samsung.com'>k.krolik@samsung.com</a>
 * @author Tomasz Scislo <a href='mailto:t.scislo@samsung.com'>t.scislo@samsung.com</a>
 * Copyright (c) 2012 Samsung Electronics All Rights Reserved.
 * @author Lukasz Jagodzinski <a href='mailto:l.jagodzinsk@samsung.com'>l.jagodzinsk@samsung.com</a>
 * Copyright (c) 2012 Samsung Electronics All Rights Reserved.
 ******************************************************************************/

var LoveMeter = (function ($) {
    var $people;

    // Display mode on contacts list
    var displayMode = {
        WEEK: 0,
        MONTH: 1,
        TOTAL: 2
    };

    // Sort order of contacts list
    var sortOrder = {
        ASCENDING: 0,
        DESCENDING: 1
    };
    // sort mode of contacts list
    var states = {
        DURATION: 0,
        SMS: 1,
        CONNECTION: 2
    };

    // Current show state (duration, sms or connection)
    var state = states.DURATION;
    // Current display mode (week, month or total)
    var dispMode = displayMode.TOTAL;
    // Current sort order (ascending or descending)
    var curSortOrder = sortOrder.DESCENDING;
    // ContactList object
    var contacts;
    // Incremented when no messages or calls were found
    var noListItems = 0;

    /**
     * Increments noListItems value when no messages or calls were found. If value is 2 (there were no connections) a popup is shown
     */
    var noListItemsFound = function () {
        noListItems += 1;
        if (noListItems > 1) {
            tizen.view.showPopup("No messages or calls found!");
        }
    };

    /**
     * Change current show state
     *
     * @param newState New state
     */
    changeState = function (newState) {
        if (state === newState) {
            if (curSortOrder === sortOrder.DESCENDING) {
                curSortOrder = sortOrder.ASCENDING;
            } else {
                curSortOrder = sortOrder.DESCENDING;
            }
        } else {
            curSortOrder = sortOrder.DESCENDING;
        }
        state = newState;
    };

    /**
     * Mark selected state on list
     */
    markSelectedState = function () {
        switch (state) {
        case states.CONNECTION:
            $('.connections').find('.lm').addClass('red');
            $('.sms').find('.lm').removeClass('red');
            $('.duration').find('.lm').removeClass('red');
            break;
        case states.DURATION:
            $('.connections').find('.lm').removeClass('red');
            $('.sms').find('.lm').removeClass('red');
            $('.duration').find('.lm').addClass('red');
            break;
        case states.SMS:
            $('.connections').find('.lm').removeClass('red');
            $('.sms').find('.lm').addClass('red');
            $('.duration').find('.lm').removeClass('red');
            break;
        default:
        }
    };

    return {
        /**
         * Initializes LoveMeter application
         */
        initialize: function () {
            $people = $('#people');

            tizen.logger = tizen.logger({
                logLevel: 3
            });
            tizen.logger.info("LoveMeter.initialize()");

            $.extend($.mobile, {
                defaultPageTransition: "flip",
                loadingMessageTextVisible: true,
                pageLoadErrorMessage: "Unable to load the page",
                pageLoadErrorMessageTheme: "d",
                touchOverflowEnabled: true,
                allowCrossDomainPages: true,
                ajaxEnabled: false
            });
            $('.exit').unbind().bind({
                click: function () {
                    if (confirm("Exit?")) {
                        tizen.application.getCurrentApplication().exit();
                    }
                }
            });

            tizen.ajax();

            tizen.application.visibility.onApplicationVisible(function () {
                createList();
            });

            /**
             * Refresh contact list
             */
            function showContacts () {
                $people.empty();
                $people.append("<li data-role='list-divider'><div><img src='./images/call.png' class='indicator'/><img src='./images/sms.png' class='indicator' /><img src='./images/duration.png' class='indicator' /></div></li>");
                for (var i = 0; i < contacts.getLength(); i++) {
                    $('#people').append(contacts.get(i).toHTML(dispMode));
                    contacts.get(i).addClickEventListener();
                }
                $people.listview('refresh');
                $('.connections').unbind().bind({
                    click: function (event) {
                        event.stopPropagation();
                        tizen.logger.info('sort by connection');
                        changeState(states.CONNECTION);
                        contacts.sortByConnections(dispMode, curSortOrder);
                        showContacts();
                        markSelectedState();
                    }
                });
                $('.sms').unbind().bind({
                    click: function (event) {
                        event.stopPropagation();
                        tizen.logger.info('sort by sms');
                        changeState(states.SMS);
                        contacts.sortBySMS(dispMode, curSortOrder);
                        showContacts();
                        markSelectedState();
                    }
                });
                $('.duration').unbind().bind({
                    click: function (event) {
                        event.stopPropagation();
                        tizen.logger.info('sort by duration');
                        changeState(states.DURATION);
                        contacts.sortByDuration(dispMode, curSortOrder);
                        showContacts();
                        markSelectedState();
                    }
                });

            }

            /**
             * Callback for new call history items found. Add found calls to appropriate contact in contacts array
             *
             * @param results Call history items
             */
            function onCallHistoryFindSuccess (results) {
                gCallHistory = results;
                tizen.logger.info(results.length + " call(s) fetched!");
                if (results.length > 0) {
                    for (var i = 0; i < results.length; i++) {
                        var conn = new connection({
                            number: results[i].remoteParties[0].remoteParty,
                            date: results[i].startTime,
                            sms: 0,
                            connection: 1,
                            duration: results[i].duration
                        });
                        contacts.add(conn);
                    }
                    resetList();

                } else {
                    noListItemsFound();
                }
            }

            /**
             * Callback for history find error
             *
             * @param error Error message
             */

            function onCallHistoryFindError (error) {
                tizen.logger.err("Error: Call history query failed: " + error.message);
                noListItemsFound();
            }

            /**
             * Callback for message items found. Add found messages to appropriate contact in contacts array
             *
             * @param messages Message items
             */

            function messageArrayCB (messages) {
                tizen.logger.info(messages.length + " message(s) fetched!");
                if (messages.length === 0) {
                    noListItemsFound();
                }
                for (var i = 0; i < messages.length; i++) {
                    if (messages[i].from !== "") {
                        var conn = new connection({
                            number: messages[i].from,
                            date: messages[i].timestamp,
                            sms: 1,
                            connection: 0,
                            duration: 0
                        });
                        contacts.add(conn);
                    }
                }
                resetList();
            }

            /**
             * Callback for message find items error
             *
             * @param error Error message
             */
            function errorCallback (error) {
                tizen.logger.err("Cannot query messages" + error.message);
                noListItemsFound();
            }

            /**
             * Callback for receiving message services object. Starts a query for all messages
             *
             * @param services Message services object
             */
            function serviceListCB (services) {
                smsService = services[0];
                // Set the abstract filter
                var filter = new tizen.AttributeFilter("type", "EXACTLY", "messaging.sms");
                smsService.messageStorage.findMessages(filter, messageArrayCB, errorCallback);
            }

            /**
             * Refresh list to default parameters
             */
            function resetList () {
                contacts.matchNames(function () {
                    changeState(states.CONNECTION);
                    curSortOrder = sortOrder.DESCENDING;
                    contacts.sortByConnections(dispMode, curSortOrder);
                    showContacts();
                    markSelectedState();
                });
            }

            // binders for state radio buttons
            $('#week').bind({
                click: function (event) {
                    event.stopPropagation();
                    dispMode = displayMode.WEEK;
                    resetList();
                }
            });

            $('#month').bind({
                click: function (event) {
                    event.stopPropagation();
                    dispMode = displayMode.MONTH;
                    resetList();
                }
            });

            $('#overall').bind({
                click: function (event) {
                    event.stopPropagation();
                    dispMode = displayMode.TOTAL;
                    resetList();
                }
            });

            /**
             * Query call and message history and create list (list is created
             * in listeners, if messages/calls are found).
             */
            function createList () {
                noListItems = 0;
                contacts = new contactList();

                // search for calls
                var filter = new tizen.AttributeFilter("type", "EXACTLY", "TEL"),
                    sortMode = new tizen.SortMode("startTime", "DESC");

                try {
                    tizen.callhistory.find(onCallHistoryFindSuccess, onCallHistoryFindError, filter, sortMode);
                } catch (err) {
                    alert("Error: Find failed: " + err.code + ": " + err.message);
                }

                // search for smses
                tizen.messaging.getMessageServices("messaging.sms", serviceListCB);
            }

            /**
             * Main function, creating/refreshing list of calls and messages
             */
            $('#main').unbind().live('pageshow', function (event) {
                createList();
            });
        },

        getDisplayMode: function () {
            return displayMode;
        },
        getSortOrder: function () {
            return sortOrder;
        }
    };
})(jQuery);
